@@ -1,13 +1,19 @@
 import sys
 from typing import (
-    cast, Dict, List, Optional, overload, Tuple, Type, TypeVar, Union,
+    cast, Dict, get_type_hints,
+    List, Optional, overload, Tuple, TYPE_CHECKING, Type, TypeVar, Union,
 )
 
 # Literal
 if sys.version_info >= (3, 8):
     from typing import Literal  # Python 3.8+
 else:
-    from typing_extensions import Literal  # Python 3.5+
+    try:
+        from typing_extensions import Literal  # Python 3.5+
+    except ImportError:
+        if not TYPE_CHECKING:
+            class Literal:
+                pass
 
 # get_origin, get_args
 if sys.version_info >= (3, 8):
@@ -31,7 +37,12 @@ elif sys.version_info >= (3, 7):
 elif sys.version_info >= (3, 6):
     from typing import GenericMeta  # type: ignore  # private API not in stubs
     from typing import _Union  # type: ignore  # private API not in stubs
-    from typing_extensions import _Literal  # type: ignore  # private API not in stubs
+    try:
+        from typing_extensions import _Literal  # type: ignore  # private API not in stubs
+    except ImportError:
+        if not TYPE_CHECKING:
+            class _Literal:
+                pass
     
     def get_origin(tp: object) -> Optional[object]:
         if isinstance(tp, GenericMeta):
@@ -55,27 +66,27 @@ elif sys.version_info >= (3, 6):
 else:
     raise ImportError('Expected Python 3.6 or later.')
 
+
 # _is_typed_dict
+_typed_dict_meta_list = []
 try:
-    from typing import _TypedDictMeta  # type: ignore  # private API not in stubs
-    
-    def _is_typed_dict(tp: object) -> bool:
-        return isinstance(tp, _TypedDictMeta)
+    from typing import _TypedDictMeta as _TypingTypedDictMeta  # type: ignore  # private API not in stubs
+    _typed_dict_meta_list.append(_TypingTypedDictMeta)
 except ImportError:
-    try:
-        from typing_extensions import _TypedDictMeta  # type: ignore  # private API not in stubs
-        
-        def _is_typed_dict(tp: object) -> bool:
-            return isinstance(tp, _TypedDictMeta)
-    except ImportError:
-        try:
-            from mypy_extensions import _TypedDictMeta  # type: ignore  # private API not in stubs
-            
-            def _is_typed_dict(tp: object) -> bool:
-                return isinstance(tp, _TypedDictMeta)
-        except ImportError:
-            def _is_typed_dict(tp: object) -> bool:
-                return False
+    pass
+try:
+    from typing_extensions import _TypedDictMeta as _TypingExtensionsTypedDictMeta  # type: ignore  # private API not in stubs
+    _typed_dict_meta_list.append(_TypingExtensionsTypedDictMeta)
+except ImportError:
+    pass
+try:
+    from mypy_extensions import _TypedDictMeta as _MypyExtensionsTypedDictMeta  # type: ignore  # private API not in stubs
+    _typed_dict_meta_list.append(_MypyExtensionsTypedDictMeta)
+except ImportError:
+    pass
+_typed_dict_metas = tuple(_typed_dict_meta_list)
+def _is_typed_dict(tp: object) -> bool:
+    return isinstance(tp, tuple(_typed_dict_metas))
 
 
 __all__ = ['trycast']
@@ -190,11 +201,22 @@ def trycast(type, value, failure=None):
     
     if _is_typed_dict(type):  # T extends TypedDict
         if isinstance(value, dict):
-            if type.__total__ and len(value) != len(type.__annotations__):
-                return failure
-            for (k, V) in type.__annotations__.items():
-                v = value.get(k, _MISSING)
-                if v is _MISSING or trycast(V, v, _FAILURE) is _FAILURE:
+            resolved_annotations = get_type_hints(type)  # resolve ForwardRefs in type.__annotations__
+            try:
+                # {typing in Python 3.9+, typing_extensions}.TypedDict
+                required_keys = type.__required_keys__
+            except AttributeError:
+                # {typing in Python 3.8, mypy_extensions}.TypedDict
+                if type.__total__:
+                    required_keys = resolved_annotations.keys()
+                else:
+                    required_keys = frozenset()
+            for (k, v) in value.items():
+                V = resolved_annotations.get(k, _MISSING)
+                if V is _MISSING or trycast(V, v, _FAILURE) is _FAILURE:
+                    return failure
+            for k in required_keys:
+                if k not in value:
                     return failure
             return cast(_T, value)
         else:
